#include "DispP.h"

/*===========================================================================*/
/* μ                                                                  */
/*===========================================================================*/

unsigned long int Disp_GetPixel(Disp disp, char * color_name)
{
  XColor c0, c1;
  XAllocNamedColor(disp->display, disp->colormap, color_name, &c0, &c1);
  return (c0.pixel);
}

Display * Disp_GetDisplay(Disp disp) { return (disp->display ); }
Window Disp_GetWindow(Disp disp) { return (disp->window ); }
int Disp_GetWidth( Disp disp) { return (disp->width ); }
int Disp_GetHeight(Disp disp) { return (disp->height); }

/*===========================================================================*/
/* ֥ȤȺ                                                  */
/*===========================================================================*/

/*---------------------------------------------------------------------------*/
/* ֥Ȥ                                                        */
/*---------------------------------------------------------------------------*/

Disp Disp_Create(char * display_name, char * background_color)
{
  Disp disp;

  disp = (Disp)malloc(sizeof(_Disp));
  if (!disp) Error("Disp_Create", "Cannot allocate memory.");

  disp->display = XOpenDisplay(display_name);
  if (disp->display == NULL) Error("Disp_Create", "Cannot open display.");

  disp->screen = DefaultScreen(disp->display);

  disp->colormap = DefaultColormap(disp->display, disp->screen);

  disp->width  = DisplayWidth( disp->display, disp->screen);
  disp->height = DisplayHeight(disp->display, disp->screen);

  disp->window = RootWindow(disp->display, disp->screen);

  disp->depth = DefaultDepth(disp->display, disp->screen);

  disp->gc_background = XCreateGC(disp->display, disp->window, 0, 0);
  disp->gc_draw = XCreateGC(disp->display, disp->window, 0, 0);

  XSetForeground(disp->display, disp->gc_background,
		 Disp_GetPixel(disp, background_color));
  XSetBackground(disp->display, disp->gc_background,
		 Disp_GetPixel(disp, background_color));
  XSetForeground(disp->display, disp->gc_draw,
		 Disp_GetPixel(disp, "red"));
  XSetBackground(disp->display, disp->gc_draw,
		 Disp_GetPixel(disp, "red"));

  XSetFunction(disp->display, disp->gc_background, GXcopy);
  XSetFunction(disp->display, disp->gc_draw, GXcopy);

  XFillRectangle(disp->display, disp->window, disp->gc_background,
		 0, 0, disp->width, disp->height);

  return (disp);
}

Disp Disp_Destroy(Disp disp)
{
  if (!disp) return (NULL);

  if (disp->gc_background) XFreeGC(disp->display, disp->gc_background);
  if (disp->gc_draw) XFreeGC(disp->display, disp->gc_draw);

  if (disp->display) XCloseDisplay(disp->display);

  free(disp);

  return (NULL);
}

/*---------------------------------------------------------------------------*/
/* եå                                                                */
/*---------------------------------------------------------------------------*/

int Disp_Flush(Disp disp)
{
  XEvent ev;

  while (XPending(disp->display)) {
    XNextEvent(disp->display, &ev);
  }
  XFlush(disp->display);

  return (0);
}

/*---------------------------------------------------------------------------*/
/* */
/*---------------------------------------------------------------------------*/

int Disp_Clear(Disp disp)
{
  XFillRectangle(disp->display, disp->window, disp->gc_background,
		 0, 0, disp->width, disp->height);
  Disp_Flush(disp);
  return (0);
}

/*---------------------------------------------------------------------------*/
/* ߤ                                                                  */
/*---------------------------------------------------------------------------*/

int Disp_DrawFilledCircle(Disp disp, GC gc, int x, int y, int r)
{
  if (x + r < 0 || disp->width  - 1 < x - r ||
      y + r < 0 || disp->height - 1 < y - r) return (0);

  XFillArc(disp->display, disp->window, gc,
	   x - r, y - r, r * 2 + 1, r * 2 + 1, 0, 360 * 64);

  return (0);
}

int Disp_DrawFilledCircles(Disp disp, GC gc, XArc * arcs, int n)
{
  XFillArcs(disp->display, disp->window, gc, arcs, n);

  return (0);
}

/*****************************************************************************/
/* ޤ                                                                  */
/*****************************************************************************/

/*****************************************************************************/
/* End of File.                                                              */
/*****************************************************************************/
