#include "GCObjP.h"

GCObj GCObj_Create(Disp disp)
{
  GCObj gc_obj;

  gc_obj = (GCObj)malloc(sizeof(_GCObj));

  gc_obj->disp = disp;
  gc_obj->gc = XCreateGC(Disp_GetDisplay(disp), Disp_GetWindow(disp), 0, 0);

  return (gc_obj);
}

GCObj GCObj_Destroy(GCObj gc_obj)
{
  if (!gc_obj) return (NULL);

  if (gc_obj->gc) XFreeGC(Disp_GetDisplay(gc_obj->disp), gc_obj->gc);

  free(gc_obj);

  return (NULL);
}

GC GCObj_GetGC(GCObj gc_obj)
{
  return (gc_obj->gc);
}

int GCObj_SetColor(GCObj gc_obj, unsigned long int color)
{
  XSetForeground(Disp_GetDisplay(gc_obj->disp), gc_obj->gc, color);
  XSetBackground(Disp_GetDisplay(gc_obj->disp), gc_obj->gc, color);

  return (0);
}

int GCObj_SetColorByName(GCObj gc_obj, char * color_name)
{
  unsigned long int color;

  color = Disp_GetPixel(gc_obj->disp, color_name);
  return (GCObj_SetColor(gc_obj, color));
}

int GCObj_SetColorByRGBColor(GCObj gc_obj, RGBColor rgb_color)
{
  unsigned long int color;

  color = RGBColor_GetColor(gc_obj->disp, rgb_color);
  return (GCObj_SetColor(gc_obj, color));
}

ObjList GCObj_CreateGCObjList(Disp disp, char * color_change,
			      int after_image_length)
{
  ObjList rgb_color_change_list;

  int counter;
  int length;
  int brightness;
  int i;
  ObjList gc_list;
  ObjList gc_list_list;
  RGBColor rgb_color;
  RGBColor rgb_color2;
  GCObj gc_obj;

  if (color_change == NULL) return (NULL);

  rgb_color_change_list =
    RGBColorChangeList_CreateFromCharacters(color_change);

  counter = 0;
  gc_list_list = ObjList_Create();

  while ((rgb_color =
	  RGBColorChangeList_GetRGBColor(rgb_color_change_list, counter))
	 != NULL) {
    counter++;
    gc_list = ObjList_Create();

    /* 1100 ͤ뤵 */
    brightness = RGBColor_GetBrightness(rgb_color);
    length = after_image_length * brightness / 100;
    if (length == 0) length = 1;

    for (i = 0; i <= length; i++) {
      rgb_color2 = RGBColor_CreateMiddleColorToBlack(rgb_color,
						     i * 100 / length);
      gc_obj = GCObj_Create(disp);
      GCObj_SetColorByRGBColor(gc_obj, rgb_color2);
      ObjList_InsertObjToEnd(gc_list, gc_obj, (ObjDestructor)GCObj_Destroy);
      RGBColor_Destroy(rgb_color2);
    }

    ObjList_InsertObjToEnd(gc_list_list, gc_list,
			   (ObjDestructor)ObjList_Destroy);

    RGBColor_Destroy(rgb_color);
  }

  ObjList_Destroy(rgb_color_change_list);

  return (gc_list_list);
}

/* End of File */
